@@ -18,12 +18,12 @@ import org.gcube.portlets.user.tdwx.client.config.Row;
 import org.gcube.portlets.user.tdwx.client.config.TableViewConfig;
 import org.gcube.portlets.user.tdwx.client.config.TabularDataGridViewConfig;
 import org.gcube.portlets.user.tdwx.client.event.CloseTableEvent;
-import org.gcube.portlets.user.tdwx.client.event.CloseTableEventHandler;
+import org.gcube.portlets.user.tdwx.client.event.CloseTableEvent.CloseTableEventHandler;
 import org.gcube.portlets.user.tdwx.client.event.GridReadyEvent;
 import org.gcube.portlets.user.tdwx.client.event.OpenTableEvent;
-import org.gcube.portlets.user.tdwx.client.event.OpenTableEventHandler;
+import org.gcube.portlets.user.tdwx.client.event.OpenTableEvent.OpenTableEventHandler;
 import org.gcube.portlets.user.tdwx.client.event.TableReadyEvent;
-import org.gcube.portlets.user.tdwx.client.event.TableReadyEventHandler;
+import org.gcube.portlets.user.tdwx.client.event.TableReadyEvent.TableReadyEventHandler;
 import org.gcube.portlets.user.tdwx.client.filter.ExtendedGridFilters;
 import org.gcube.portlets.user.tdwx.client.filter.FiltersGenerator;
 import org.gcube.portlets.user.tdwx.client.model.grid.DataRowColumnConfig;
@@ -51,7 +51,6 @@ import com.google.gwt.event.logical.shared.SelectionHandler;
 import com.google.gwt.event.shared.EventBus;
 import com.google.gwt.http.client.RequestBuilder;
 import com.sencha.gxt.core.client.Style.SelectionMode;
-import com.sencha.gxt.core.client.resources.ThemeStyles;
 import com.sencha.gxt.core.client.util.Margins;
 import com.sencha.gxt.data.client.loader.HttpProxy;
 import com.sencha.gxt.data.shared.ListStore;
@@ -72,15 +71,13 @@ import com.sencha.gxt.widget.core.client.event.HeaderMouseDownEvent.HeaderMouseD
 import com.sencha.gxt.widget.core.client.grid.ColumnConfig;
 import com.sencha.gxt.widget.core.client.grid.ColumnModel;
 import com.sencha.gxt.widget.core.client.grid.Grid;
-import com.sencha.gxt.widget.core.client.grid.LiveGridView;
-import com.sencha.gxt.widget.core.client.grid.LiveToolItem;
 import com.sencha.gxt.widget.core.client.grid.filters.Filter;
 import com.sencha.gxt.widget.core.client.menu.Item;
 import com.sencha.gxt.widget.core.client.menu.Menu;
 import com.sencha.gxt.widget.core.client.menu.MenuItem;
 import com.sencha.gxt.widget.core.client.menu.SeparatorMenuItem;
 import com.sencha.gxt.widget.core.client.selection.CellSelection;
-import com.sencha.gxt.widget.core.client.toolbar.ToolBar;
+import com.sencha.gxt.widget.core.client.toolbar.PagingToolBar;
 
 /**
  * 
@@ -92,45 +89,50 @@ import com.sencha.gxt.widget.core.client.toolbar.ToolBar;
  */
 public class TabularDataXGridPanel extends ContentPanel {
 
-	protected int tdSessionId;
+	private static final int PAGINGDIMENSION = 300;
 
-	protected EventBus eventBus;
+	private int tdSessionId;
 
-	protected List<MenuItem> headerColumnMenuItems;
+	private EventBus eventBus;
 
-	protected com.google.web.bindery.event.shared.EventBus externalBus;
+	private List<MenuItem> headerColumnMenuItems;
 
-	protected ListStore<DataRow> store;
-	protected Grid<DataRow> grid;
-	protected ToolBar toolBar;
-	// protected GridInlineEditing<DataRow> editing;
-	protected TableDefinition tableDefinition;
+	private com.google.web.bindery.event.shared.EventBus externalBus;
+
+	private ListStore<DataRow> store;
+	private Grid<DataRow> grid;
 	
-	protected VerticalLayoutContainer container;
+	private TableDefinition tableDefinition;
 
-	protected ExtendedLiveGridView liveGridView;
+	private VerticalLayoutContainer container;
 
-	protected DataRowPagingReader reader;
+	//private ExtendedLiveGridView<DataRow> liveGridView;
 
-	protected PagingLoader<FilterPagingLoadConfig, PagingLoadResult<DataRow>> loader;
+	private DataRowPagingReader reader;
 
-	protected ExtendedGridFilters<DataRow> gridFilters;
-	
-	protected ArrayList<StaticFilterInformation> staticFilters;
+	private PagingLoader<FilterPagingLoadConfig, PagingLoadResult<DataRow>> loader;
 
-	protected TableViewConfig tableViewConfig;
+	private ExtendedGridFilters<DataRow> gridFilters;
 
-	protected Menu contextMenu;
+	private ArrayList<StaticFilterInformation> staticFilters;
 
-	protected TableDefinition currentTableDefinition;
-	protected Map<String, ColumnKey> keys;
+	private TableViewConfig tableViewConfig;
+
+	private Menu contextMenu;
+
+	private Map<String, ColumnKey> keys;
 
 	private String visibleOnlyColumn;
-	
-	protected boolean errorNotColored=false;
-	
-	
 
+	private boolean errorNotColored = false;
+
+	private ColumnModel<DataRow> columnModel;
+
+	private GridAndCellSelectionModel<DataRow> sm;
+
+	private PagingToolBarX pagingToolBar;
+
+	
 	/**
 	 * @param eventBus
 	 */
@@ -141,13 +143,13 @@ public class TabularDataXGridPanel extends ContentPanel {
 		setBodyBorder(false);
 		setBorders(false);
 		setHeaderVisible(false);
-		forceLayoutOnResize=true;
+		forceLayoutOnResize = true;
 		setResize(true);
 		bindEventBus();
 		container = new VerticalLayoutContainer();
 		container.setBorders(false);
-		add(container,new MarginData(0));
-		//setWidget(container);
+		add(container, new MarginData(0));
+	
 	}
 
 	/**
@@ -212,10 +214,11 @@ public class TabularDataXGridPanel extends ContentPanel {
 	 * 
 	 * @param staticFilters
 	 */
-	public void setStaticFilters(ArrayList<StaticFilterInformation> staticFilters) {
+	public void setStaticFilters(
+			ArrayList<StaticFilterInformation> staticFilters) {
 		this.staticFilters = staticFilters;
 	}
-	
+
 	/**
 	 * Returns the selected row.
 	 * 
@@ -272,16 +275,15 @@ public class TabularDataXGridPanel extends ContentPanel {
 
 		return null;
 	}
-	
-	
 
 	/**
 	 * 
 	 * 
-	 * @return Selected Row as RowRaw
+	 * @return Selected Rows as List<RowRaw>
 	 */
-	public RowRaw getSelectedRowAsRaw() {
+	public ArrayList<RowRaw> getSelectedRowsAsRaw() {
 		if (grid != null && grid.getSelectionModel() != null) {
+			ArrayList<RowRaw> listRowRaw = new ArrayList<RowRaw>();
 			List<DataRow> dataRows = grid.getSelectionModel()
 					.getSelectedItems();
 			List<Row> rows = new ArrayList<Row>();
@@ -291,43 +293,40 @@ public class TabularDataXGridPanel extends ContentPanel {
 				}
 			}
 
-			RowRaw r = null;
-			
-			if (rows.size() > 0) {
+			for (Row row : rows) {
 				HashMap<String, String> map = new HashMap<String, String>();
 				ColumnModel<DataRow> columnModel = grid.getColumnModel();
 				List<ColumnConfig<DataRow, ?>> columns = columnModel
 						.getColumns();
-				Row rowFirst = rows.get(0);
 				DataRowColumnConfig<?> columnDataRow = null;
-				String rowId=null;
+				String rowId = null;
 				for (ColumnConfig<DataRow, ?> col : columns) {
 					columnDataRow = (DataRowColumnConfig<?>) col;
 					ColumnDefinition colDef = columnDataRow.getDefinition();
 					if (colDef != null) {
 						String value;
-						if (colDef.getColumnDataType()
-								.compareTo("Date") == 0) {
-							value = rowFirst.getFieldAsDate(colDef
+						if (colDef.getColumnDataType().compareTo("Date") == 0) {
+							value = row.getFieldAsDate(colDef
 									.getColumnLocalId());
 						} else {
-							value = rowFirst.getFieldAsText(colDef
+							value = row.getFieldAsText(colDef
 									.getColumnLocalId());
 						}
 						map.put(colDef.getColumnLocalId(), value);
 						ColumnType ctype = colDef.getType();
 						if (ctype == ColumnType.COLUMNID) {
-							rowId=value;
+							rowId = value;
 						}
-					
+
 					}
 				}
-				if(rowId!=null && !rowId.isEmpty() && map.size()>0){
-					r=new RowRaw(rowId,map);
+				if (rowId != null && !rowId.isEmpty() && map.size() > 0) {
+					RowRaw rr = new RowRaw(rowId, map);
+					listRowRaw.add(rr);
 				}
 
 			}
-			return r;
+			return listRowRaw;
 		}
 
 		return null;
@@ -352,7 +351,7 @@ public class TabularDataXGridPanel extends ContentPanel {
 				List<ColumnConfig<DataRow, ?>> columns = columnModel
 						.getColumns();
 
-				// Retrive ColumnId
+				// Retrieve ColumnId
 				DataRowColumnConfig<?> columnId = null;
 				boolean columnIdRetrieved = false;
 				for (ColumnConfig<DataRow, ?> col : columns) {
@@ -367,7 +366,7 @@ public class TabularDataXGridPanel extends ContentPanel {
 					Log.debug("ColumnId Retrieved");
 					ColumnDefinition columnIdSelected = columnId
 							.getDefinition();
-					// Retrive ColumnSelected
+					// Retrieve ColumnSelected
 					ColumnConfig<DataRow, ?> colSelected = columns.get(cell
 							.getCell());
 					DataRowColumnConfig<?> cSelected = (DataRowColumnConfig<?>) colSelected;
@@ -416,7 +415,7 @@ public class TabularDataXGridPanel extends ContentPanel {
 		visibleOnlyColumn = columnLocalId;
 
 	}
-	
+
 	/**
 	 * 
 	 * @return
@@ -424,16 +423,16 @@ public class TabularDataXGridPanel extends ContentPanel {
 	public boolean isErrorNotColored() {
 		return errorNotColored;
 	}
-	
+
 	/**
 	 * 
-	 * @param errorNotColored if true set background withe for rows with error
+	 * @param errorNotColored
+	 *            if true set background withe for rows with error
 	 */
 	public void setErrorNotColored(boolean errorNotColored) {
-		Log.debug("ErrorNotColored set :"+errorNotColored);
+		Log.debug("ErrorNotColored set :" + errorNotColored);
 		this.errorNotColored = errorNotColored;
 	}
-	
 
 	/**
 	 * 
@@ -448,15 +447,14 @@ public class TabularDataXGridPanel extends ContentPanel {
 			List<ColumnConfig<DataRow, ?>> columns = columnModel.getColumns();
 
 			List<ColumnConfig<DataRow, ?>> columnsNew = new ArrayList<ColumnConfig<DataRow, ?>>();
-			// Retrive Column with conlumnLocalId
+			
 			DataRowColumnConfig<?> columnTarget = null;
-			// boolean columnTargetRetrieved = false;
 			for (ColumnConfig<DataRow, ?> col : columns) {
 				columnTarget = (DataRowColumnConfig<?>) col;
 				String columnLocal = columnTarget.getDefinition()
 						.getColumnLocalId();
 				if (visibleOnlyColumn.compareTo(columnLocal) == 0) {
-					// columnTargetRetrieved = true;
+					
 					columnTarget.setHidden(false);
 					columnsNew.add(columnTarget);
 				} else {
@@ -486,7 +484,7 @@ public class TabularDataXGridPanel extends ContentPanel {
 
 			List<ColumnConfig<DataRow, ?>> columns = columnModel.getColumns();
 
-			// Retrive Column with conlumnLocalId
+			// Retrieve Column with conlumnLocalId
 			DataRowColumnConfig<?> columnTarget = null;
 			boolean columnTargetRetrieved = false;
 			for (ColumnConfig<DataRow, ?> col : columns) {
@@ -562,11 +560,11 @@ public class TabularDataXGridPanel extends ContentPanel {
 						rows.add(rowS);
 					}
 				} else {
-					Log.debug("no selected rows retrived");
+					Log.debug("no selected rows retrieved");
 				}
 				return rows;
 			} else {
-				Log.debug("no COLUMNID retrived");
+				Log.debug("no COLUMNID retrieved");
 				return null;
 			}
 
@@ -610,7 +608,6 @@ public class TabularDataXGridPanel extends ContentPanel {
 		Log.trace("table ready, setting grid up");
 		mask("Loading table " + definition.getName() + "... ");
 		setupGrid(definition);
-		this.currentTableDefinition = definition;
 		keys = new HashMap<String, ColumnKey>();
 		for (ColumnDefinition column : definition.getColumnsAsList())
 			keys.put(column.getColumnLocalId(), column.getKey());
@@ -620,8 +617,7 @@ public class TabularDataXGridPanel extends ContentPanel {
 	protected void doCloseTable() {
 		mask();
 		grid = null;
-		this.currentTableDefinition = null;
-		this.keys.clear();
+		keys.clear();
 		container.clear();
 		unmask();
 	}
@@ -649,7 +645,7 @@ public class TabularDataXGridPanel extends ContentPanel {
 					}
 				};
 			}
-			;
+
 			grid.addHeaderMouseDownHandler(handler);
 		}
 	}
@@ -687,18 +683,49 @@ public class TabularDataXGridPanel extends ContentPanel {
 						.generateConfiguration(columnDefinition));
 		}
 
-		// IdentityValueProvider<DataRow> identity = new
-		// IdentityValueProvider<DataRow>();
-		// LiveRowNumberer<DataRow> numberer = new
-		// LiveRowNumberer<DataRow>(identity);
-		// columnsConfig.add(numberer);
 
-		ColumnModel<DataRow> columnModel = new ColumnModel<DataRow>(
-				columnsConfig);
+		columnModel = new ColumnModel<DataRow>(columnsConfig);
 
 		columnModel = checkOnlyColumn(columnModel);
 
+		/*
+		columnModel
+				.addColumnMoveHandler(new ColumnMoveEvent.ColumnMoveHandler() {
+
+					@Override
+					public void onColumnMove(ColumnMoveEvent event) {
+						int columnIndex = event.getIndex();
+						@SuppressWarnings("unchecked")
+						ColumnConfig<DataRow, ?> col = (ColumnConfig<DataRow, ?>) event
+								.getColumnConfig();
+						DataRowColumnConfig<?> columnDataRow = (DataRowColumnConfig<?>) col;
+						ColumnDefinition colDef = columnDataRow.getDefinition();
+
+						Log.debug("Column Reordering", "Index: " + columnIndex
+								+ " Label: " + colDef.getLabel()
+								+ " Position: " + colDef.getPosition()
+								+ " ColumnId: " + colDef.getColumnLocalId());
+
+						if (colDef.isViewColumn()) {
+							Info.display("Attention",
+									"The view columns can not be moved");
+							// ColumnHeader<DataRow>
+							// colHeader=grid.getView().getHeader();
+
+							// event.getSource().moveColumn(columnIndex,
+							// colDef.getPosition());
+						} else {
+							ColumnsReorderingConfig columnsReorderingConfig = new ColumnsReorderingConfig(
+									columnIndex, colDef);
+							ColumnsReorderingEvent columnsReorderingEvent = new ColumnsReorderingEvent(
+									columnsReorderingConfig);
+							eventBus.fireEvent(columnsReorderingEvent);
+						}
+					}
+				});
+		*/
 		if (grid == null) {
+			Log.debug("Setup reader");
 
 			reader = new DataRowPagingReader(tableDefinition);
 
@@ -707,13 +734,13 @@ public class TabularDataXGridPanel extends ContentPanel {
 					path);
 			builder.setHeader(ServletParameters.TD_SESSION_ID,
 					String.valueOf(tdSessionId));
-			// builder.setHeader("Content-Type",
-			// "application/x-www-form-urlencoded");
+			
 
 			HttpProxy<FilterPagingLoadConfig> proxy = new HttpProxy<FilterPagingLoadConfig>(
 					builder);
 			proxy.setWriter(new PagingLoadUrlEncoder(staticFilters));
 
+
 			loader = new PagingLoader<FilterPagingLoadConfig, PagingLoadResult<DataRow>>(
 					proxy, reader) {
 				@Override
@@ -722,46 +749,38 @@ public class TabularDataXGridPanel extends ContentPanel {
 				}
 			};
 
-			/*
-			 * HttpProxy<PagingLoadConfig> proxy = new
-			 * HttpProxy<PagingLoadConfig>( builder); proxy.setWriter(new
-			 * PagingLoadUrlEncoder());
-			 * 
-			 * loader = new PagingLoader<PagingLoadConfig,
-			 * PagingLoadResult<DataRow>>( proxy, reader) {
-			 * 
-			 * @Override protected PagingLoadConfig newLoadConfig() { return new
-			 * PagingLoadConfigBean(); } };
-			 */
-
 			loader.setRemoteSort(true);
 
 			loader.addLoadHandler(new LoadResultListStoreBinding<FilterPagingLoadConfig, DataRow, PagingLoadResult<DataRow>>(
 					store));
 
-			liveGridView = new ExtendedLiveGridView();
+			pagingToolBar = new PagingToolBarX(PAGINGDIMENSION);
+			pagingToolBar.getElement().getStyle()
+					.setProperty("borderBottom", "none");
+			pagingToolBar.fixPageTextWidth();
+			pagingToolBar.bind(loader);
 
-			// liveGridView.setRowHeight(19);
 
-			grid = new Grid<DataRow>(store, columnModel);
-			GridAndCellSelectionModel<DataRow> sm = new GridAndCellSelectionModel<DataRow>();
+			grid = new Grid<DataRow>(store, columnModel) {
+				@Override
+				protected void onAfterFirstAttach() {
+					super.onAfterFirstAttach();
+					Scheduler.get().scheduleDeferred(new ScheduledCommand() {
+						@Override
+						public void execute() {
+							loader.load();
+						}
+					});
+				}
+			};
+			sm = new GridAndCellSelectionModel<DataRow>();
 
 			grid.setSelectionModel(sm);
 			grid.setLoadMask(true);
 			grid.setLoader(loader);
-			grid.setView(liveGridView);
-			grid.setBorders(false);
 			
-			// TODO disabled editing rows
-			/*
-			 * editing = new GridInlineEditing<DataRow>(grid);
-			 * editing.setClicksToEdit(ClicksToEdit.TWO);
-			 * 
-			 * for (ColumnConfig<DataRow, ?> c : columnsConfig) {
-			 * ColumnConfigGenerator.setEditor(editing, c); }
-			 */
-
-			// numberer.initPlugin(grid);
+			grid.setBorders(false);
+			grid.setColumnReordering(false);
 
 			// Filter
 			ArrayList<Filter<DataRow, ?>> filters = FiltersGenerator
@@ -777,19 +796,9 @@ public class TabularDataXGridPanel extends ContentPanel {
 
 			//
 			container.add(grid, new VerticalLayoutData(1, 1, new Margins(0)));
+			container.add(pagingToolBar, new VerticalLayoutData(1, -1));
+
 			
-			toolBar = new ToolBar();
-			toolBar.add(new LiveToolItem(grid));
-			toolBar.addStyleName(ThemeStyles.getStyle().borderTop());
-			toolBar.getElement().getStyle().setProperty("borderBottom", "none");
-
-			container.add(toolBar,
-					new VerticalLayoutData(1, 24, new Margins(0)));
-
-			container.forceLayout();
-
-			loader.load(0, liveGridView.getCacheSize());
-
 			if (contextMenu != null) {
 				grid.setContextMenu(contextMenu);
 			} else
@@ -800,35 +809,29 @@ public class TabularDataXGridPanel extends ContentPanel {
 			}
 
 		} else {
-			// TODO we need to manually reset the sort
-			loader.clearSortInfo();
+			Log.debug("Setup grid not null");
+			
 			reader.setDefinition(tableDefinition);
+			loader.clearSortInfo();
+
+			loader.addLoadHandler(new LoadResultListStoreBinding<FilterPagingLoadConfig, DataRow, PagingLoadResult<DataRow>>(
+					store));
 
 			grid.reconfigure(store, columnModel);
+			
 
+			// Filter
 			ArrayList<Filter<DataRow, ?>> filters = FiltersGenerator
 					.generate(columnModel);
-			gridFilters.removeAll();
 
+			
 			Log.debug("Filters: " + filters.size());
 			for (Filter<DataRow, ?> filterGeneric : filters) {
 				gridFilters.addFilter(filterGeneric);
 			}
 
-			// TODO Disabled editing rows
-			//
-			// editing = new GridInlineEditing<DataRow>(grid);
-			// editing.setClicksToEdit(ClicksToEdit.TWO);
-			//
-			// for (ColumnConfig<DataRow, ?> c : columnsConfig) {
-			// ColumnConfigGenerator.setEditor(editing, c); }
-			//
-
-			// TODO workaround: the loader is called only if the scroll bar is
-			// in the middle
-			// if (loader.getOffset() == 0)
-			// loader.load();
-			loader.load(0, liveGridView.getCacheSize());
+			loader.load();
+			
 		}
 
 		if (tableViewConfig != null) {
@@ -840,31 +843,21 @@ public class TabularDataXGridPanel extends ContentPanel {
 			}
 		} else {
 			Log.debug("Use default grid View");
-			tableViewConfig= new TableViewConfig();
-			Log.debug("Error Not Colored: "+errorNotColored);
-			DefaultRowStyle rowStyle=new DefaultRowStyle(errorNotColored);
-			
+			tableViewConfig = new TableViewConfig();
+			Log.debug("Error Not Colored: " + errorNotColored);
+			DefaultRowStyle rowStyle = new DefaultRowStyle(errorNotColored);
+
 			tableViewConfig.setRowStyleProvider(rowStyle);
 			grid.getView().setViewConfig(
 					new TabularDataGridViewConfig(tableViewConfig,
 							tableDefinition));
-			//	grid.getView().setViewConfig(null);
+			
 		}
 
 		container.forceLayout();
 
-		Scheduler.get().scheduleDeferred(new ScheduledCommand() {
-
-			public void execute() {
-				container.forceLayout();
-			}
-		});
-
 		eventBus.fireEvent(new GridReadyEvent());
-		
-		
-		
-		
+
 	}
 
 	/**
@@ -926,6 +919,7 @@ public class TabularDataXGridPanel extends ContentPanel {
 			}
 		};
 		Log.debug("Created Handler");
+
 		grid.addHeaderContextMenuHandler(headerContextMenuHandler);
 		Log.debug("Header Column Menu Added");
 
@@ -953,7 +947,6 @@ public class TabularDataXGridPanel extends ContentPanel {
 		return columnName;
 	}
 
-	
 	/**
 	 * 
 	 * @param i
@@ -975,41 +968,5 @@ public class TabularDataXGridPanel extends ContentPanel {
 		}
 		return columnLocalId;
 	}
-	
-	
-	/**
-	 * 
-	 * @param rows
-	 */
-	public void addRow(ArrayList<String> rows) {
-		// TODO Disabled AddROW
-		/*
-		 * if (grid != null && editing != null) {
-		 * 
-		 * List<ColumnKey> keys = tableDefinition.getKeys();
-		 * 
-		 * DataRow dataRow = new DataRow(keys.size()); for (ColumnKey key :
-		 * keys) { dataRow.set(key, null); }
-		 * 
-		 * editing.cancelEditing(); int pos = 0; if (rows != null && rows.size()
-		 * > 0) { pos = new Integer(rows.get(0)); }
-		 * 
-		 * store.add(pos, dataRow);
-		 * 
-		 * int row = store.indexOf(dataRow); editing.startEditing(new
-		 * GridCell(row, pos)); }
-		 */
-	}
-
-	protected class ExtendedLiveGridView extends LiveGridView<DataRow> {
-		// TODO bug in gxt3 3.0.0 fixed in future
-
-		@Override
-		public void refresh(boolean headerToo) {
-			preventScrollToTopOnRefresh = true;
-			super.refresh(headerToo);
-		}
-
-	};
 
 }
